unit Unit1;

interface


uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, PDF, PtblRV, RVReport, RVStyle, ShellApi, ExtCtrls, CRVData, RVTable,
  RVScroll, RichView, RVEdit, RVItem;

  const AlienCharsets = [ARABIC_CHARSET, HEBREW_CHARSET, SHIFTJIS_CHARSET,
    HANGEUL_CHARSET, JOHAB_CHARSET, GB2312_CHARSET, CHINESEBIG5_CHARSET,
    VIETNAMESE_CHARSET, THAI_CHARSET];

type


  TForm1 = class(TForm)
    pdf: TPDFDocument;
    btnSave: TButton;
    rvs: TRVStyle;
    rvrh: TRVReportHelper;
    btnOpen: TButton;
    od: TOpenDialog;
    sd: TSaveDialog;
    GroupBox1: TGroupBox;
    Label1: TLabel;
    cmbPageSize: TComboBox;
    rgPageOrientation: TRadioGroup;
    GroupBox2: TGroupBox;
    Label2: TLabel;
    txtLeft: TEdit;
    Label3: TLabel;
    txtTop: TEdit;
    Label4: TLabel;
    txtRight: TEdit;
    Label5: TLabel;
    txtBottom: TEdit;
    lblStatus: TLabel;
    Image1: TImage;
    Image2: TImage;
    Button1: TButton;
    Button2: TButton;
    procedure btnSaveClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure btnOpenClick(Sender: TObject);
    procedure Image1Click(Sender: TObject);
    procedure Image2Click(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
  private
    { Private declarations }
    function GetMargin(edit: TEdit): Integer;
    function GetPageSize: TPDFPageSize;
    function GetPageOrientation: TPDFPageOrientation;
    procedure ShowText(s: String; RedrawForm: Boolean);
    procedure ShowError(s: String);

    procedure RemoveAlienText(RVData: TCustomRVData);

    procedure ConvertRVFromUnicode(RVData: TCustomRVData);
    procedure ConvertFromUnicode(rv: TCustomRichView; Charset: TFontCharset);


  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}
{------------------------------------------------------------------------------}
procedure TForm1.FormCreate(Sender: TObject);
begin
  rvrh.RichView.Style := rvs;
  rvrh.RichView.RTFReadProperties.ParaStyleMode := rvrsAddIfNeeded;
  rvrh.RichView.RTFReadProperties.TextStyleMode := rvrsAddIfNeeded;
  rvrh.RichView.RTFReadProperties.UnicodeMode := rvruMixed;
  
  cmbPageSize.ItemIndex := 1;
end;

procedure TForm1.btnOpenClick(Sender: TObject);
begin
  rvrh.RichView.Clear;
  rvrh.RichView.DeleteUnusedStyles(True,True,True);
  btnSave.Enabled := False;
  if od.Execute then begin
    Screen.Cursor := crHourGlass;
    ShowText('Loading RTF file...', True);
    if rvrh.RichView.LoadRTF(od.FileName) then
      btnSave.Enabled := True
    else
      ShowError('Load Error');
    ShowText('', False);
    Screen.Cursor := crDefault;
  end;
end;

procedure TForm1.btnSaveClick(Sender: TObject);
var i, LeftMargin, TopMargin, RightMargin, BottomMargin, DocWidth, DocHeight: Integer;
  pt: TPoint;
const MinDocSize = 50;
begin
  if not sd.Execute then
    exit;
  //ConvertFromUnicode(rvrh.RichView, SHIFTJIS_CHARSET);
  //rvrh.RichView.SaveRTF('C:\tesssss.rtf', False); 
  LeftMargin   := GetMargin(txtLeft);
  TopMargin    := GetMargin(txtTop);
  RightMargin  := GetMargin(txtRight);
  BottomMargin := GetMargin(txtBottom);

  pdf.Resolution := Screen.PixelsPerInch;
  pdf.FileName := sd.FileName;

  pdf.BeginDoc;

  pdf.Page[0].Size := GetPageSize;
  pdf.Page[0].Orientation := GetPageOrientation;

  DocWidth  := pdf.Page[0].Width - (LeftMargin+RightMargin);
  DocHeight := pdf.Page[0].Height - (TopMargin+BottomMargin);

  if (DocWidth<MinDocSize) then begin
    ShowError('Horizontal margins are too large');
    txtLeft.SetFocus;
    pdf.Abort;
    exit;
  end;
  if (DocHeight<MinDocSize) then begin
    ShowError('Vertical margins are too large');
    txtTop.SetFocus;
    pdf.Abort;
    exit;
  end;

  Screen.Cursor := crHourGlass;
  try
    ShowText('Formatting...', True);
    //pdf.Canvas.Font.Charset := SHIFTJIS_CHARSET;
    rvrh.Init(pdf.Canvas, DocWidth);
    while rvrh.FormatNextPage(DocHeight) do;
    for i := 0 to rvrh.PagesCount-1 do begin
      ShowText(Format('Collecting PDF pages (%d of %d)... ', [i, rvrh.PagesCount]), False);
      if i>0 then begin
        pdf.NewPage;
        pdf.Page[i].Size        := pdf.Page[0].Size;
        pdf.Page[i].Orientation := pdf.Page[0].Orientation;
      end;
      // 3 lines below are used instead of
      // rvrh.DrawPageAt(LeftMargin, TopMargin, i+1, pdf.Canvas, False, DocHeight);
      // In newer version of LLPDFLib DrawPageAt must work too

      SetViewportOrgEx(pdf.Canvas.Handle, LeftMargin, TopMargin, @pt);
      rvrh.DrawPage(i+1, pdf.Canvas, False, DocHeight);
      SetViewportOrgEx(pdf.Canvas.Handle, pt.x, pt.y, nil);      
    end;
    ShowText('Writing PDF file...', True);
    pdf.EndDoc;
  finally
    Screen.Cursor := crDefault;
    ShowText('', False);
  end;
end;

{------------------------------------------------------------------------------}
// Returns margin, pixels
function TForm1.GetMargin(edit: TEdit): Integer;
var mm: Integer;
begin
  mm := StrToIntDef(edit.Text, 0);
  edit.Text := IntToStr(mm);
  Result := MulDiv(mm, 5*Screen.PixelsPerInch, 127);
end;
{------------------------------------------------------------------------------}
function TForm1.GetPageOrientation: TPDFPageOrientation;
begin
  Result := TPDFPageOrientation(rgPageOrientation.ItemIndex);
end;
{------------------------------------------------------------------------------}
function TForm1.GetPageSize: TPDFPageSize;
begin
  Result := TPDFPageSize(cmbPageSize.ItemIndex+1);
end;
{------------------------------------------------------------------------------}
procedure TForm1.ShowText(s: String; RedrawForm: Boolean);
begin
  if s='' then
    s := 'Ready';
  lblStatus.Caption := s;
  if RedrawForm then
    Update
  else
    lblStatus.Update;
end;
{------------------------------------------------------------------------------}
procedure TForm1.ShowError(s: String);
begin
  Application.MessageBox(PChar(s), 'Error', MB_OK or MB_ICONSTOP);
end;
{------------------------------------------------------------------------------}
procedure TForm1.Image1Click(Sender: TObject);
begin
  ShellExecute(0, 'open', 'http://www.llion.net/?From=RTF2PDF', nil, nil, SW_SHOW);
end;

procedure TForm1.Image2Click(Sender: TObject);
begin
  ShellExecute(0, 'open', 'http://www.trichview.com/?From=RTF2PDF', nil, nil, SW_SHOW);
end;


procedure TForm1.Button1Click(Sender: TObject);
begin
  ShowMessage(IntToStr(rvs.TextStyles.Count));
end;

procedure TForm1.Button2Click(Sender: TObject);
begin
  RemoveAlienText(rvrh.RichView.RVData);
//  rvrh.RichView.ClearUndo();
  rvrh.RichView.Format();

end;

procedure TForm1.RemoveAlienText(RVData: TCustomRVData);
var i,r,c: Integer;
    table: TRVTableItemInfo;
    PageBreak, BR, SameAsPrev: Boolean;
begin
  i := RVData.ItemCount-1;
  while i>=0 do begin
    if (RVData.GetItemStyle(i)>=0) and
       (RVData.GetRVStyle.TextStyles[RVData.GetItemStyle(i)].Charset in
AlienCharsets) then begin
      SameAsPrev := RVData.GetItem(i).SameAsPrev;
      PageBreak := RVData.GetItem(i).PageBreakBefore;
      BR := RVData.GetItem(i).BR;
      RVData.DeleteItems(i, 1);
      if (i>0) and (RVData.GetItemStyle(i-1)=rvsListMarker) and
         ((i=RVData.ItemCount) or RVData.IsFromNewLine(i)) then begin
        dec(i);
        RVData.DeleteItems(i, 1);
        end
      else if not SameAsPrev and (i<RVData.ItemCount) and
        not RVData.IsFromNewLine(i) then begin
        RVData.GetItem(i).SameAsPrev := False;
        RVData.GetItem(i).BR := BR;
        RVData.GetItem(i).PageBreakBefore := PageBreak;
      end;
      end
    else if (RVData.GetItemStyle(i)=rvsTable) then begin
      table := TRVTableItemInfo(RVData.GetItem(i));
      for r := 0 to table.Rows.Count-1 do
        for c := 0 to table.Rows[r].Count-1 do
          if table.Cells[r,c]<>nil then
            RemoveAlienText(table.Cells[r,c].GetRVData);
    end;
    dec(i);
  end;


end;

procedure TForm1.ConvertRVFromUnicode(RVData: TCustomRVData);
var i,r,c, StyleNo: Integer;
    table: TRVTableItemInfo;
begin
  for i := 0 to RVData.ItemCount-1 do begin
    StyleNo := RVData.GetItemStyle(i);
    if StyleNo>=0 then begin
      if RVData.GetRVStyle.TextStyles[StyleNo].Unicode then begin
        RVData.SetItemText(i, RVData.GetItemTextA(i));
        Exclude(RVData.GetItem(i).ItemOptions, rvioUnicode);
      end;
      end
    else if RVData.GetItemStyle(i)=rvsTable then begin
      table := TRVTableItemInfo(RVData.GetItem(i));
      for r := 0 to table.Rows.Count-1 do
        for c := 0 to table.Rows[r].Count-1 do
          if table.Cells[r,c]<>nil then
            ConvertRVFromUnicode(table.Cells[r,c].GetRVData);
    end;
  end;


end;

procedure TForm1.ConvertFromUnicode(rv: TCustomRichView;
  Charset: TFontCharset);
var i: Integer;
begin
  for i := 0 to rv.Style.TextStyles.Count-1 do
    rv.Style.TextStyles[i].Charset := Charset;
  ConvertRVFromUnicode(rv.RVData);
  for i := 0 to rv.Style.TextStyles.Count-1 do
    rv.Style.TextStyles[i].Unicode := False;
end;


end.

