unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, RVStyle, RVScroll, RichView, ComCtrls, RVTable;

type
  TForm1 = class(TForm)
    rv: TRichView;
    RVStyle1: TRVStyle;
    tb: TTrackBar;
    rv2: TRichView;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure tbChange(Sender: TObject);
  private
    { Private declarations }
    StreamList: TList;
  public
    { Public declarations }
    function GetFirstTableItemNo: Integer;
    procedure ExtractLinesFromTableColumn(table: TRVTableItemInfo; Col: Integer);
    procedure FillStreamList;
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

{ TForm1 }

{
  Initial property settings:
  - both rv and rv2 are set to "Allow adding styles dynamically mode"
    (right click RichView, choose "Settings" in the context menu)
  - both rv and rv2 are linked to the same RVStyle, but this is not
    important here (they can be linked to different RVStyles)
}


procedure TForm1.FormCreate(Sender: TObject);
begin
  StreamList := TList.Create;
  FillStreamList;
end;

procedure TForm1.FormDestroy(Sender: TObject);
var i: Integer;
begin
  for i := 0 to StreamList.Count-1 do
    TStream(StreamList.Items[i]).Free;
  StreamList.Free;
end;

// searching for the first table
function TForm1.GetFirstTableItemNo: Integer;
var i: Integer;
begin
  Result := -1;
  for i := 0 to rv.ItemCount-1 do
    if rv.GetItemStyle(i)=rvsTable then begin
      Result := i;
      exit;
    end;
end;

procedure TForm1.ExtractLinesFromTableColumn(table: TRVTableItemInfo;
  Col: Integer);
var Row, i, FirstParaItemNo: Integer;
    Cell: TRVTableCellData;
    {....................................................}
    procedure AddStream(FirstParaItemNo, EndParaItemNo: Integer);
    var Stream: TMemoryStream;
    begin
      rv.RVData.AssignChosenRVData(Cell, table); // well, this line is not documented
        // but it's required when selecting cell in TRichView
      Cell.SetSelectionBounds(FirstParaItemNo, Cell.GetOffsBeforeItem(FirstParaItemNo),
        EndParaItemNo, Cell.GetOffsAfterItem(EndParaItemNo));
      Stream := TMemoryStream.Create;
      rv.SaveRVFToStream(Stream, True);
      StreamList.Add(Stream);
    end;
    {....................................................}
begin
  // iterating rows
  for Row := 0 to table.Rows.Count-1 do begin
    Cell := table.Cells[Row, Col];
    if Cell=nil then
      continue;
    // iterating items in cell
    FirstParaItemNo := -1;
    for i := 0 to Cell.ItemCount-1 do begin
      if Cell.IsParaStart(i) then begin
        if i>0 then
          AddStream(FirstParaItemNo, i-1);
        FirstParaItemNo := i;
      end
    end;
    AddStream(FirstParaItemNo, Cell.ItemCount-1);
  end;
  // removing selection
  rv.Deselect;
end;

procedure TForm1.FillStreamList;
var TableItemNo: Integer;
    table: TRVTableItemInfo;
begin
  rv.LoadRTF(ExtractFilePath(Application.ExeName)+'table.rtf');
  rv.Format;
  TableItemNo := GetFirstTableItemNo;
  if TableItemNo=-1 then begin
    Application.MessageBox('No table!', nil, 0);
    exit;
  end;
  table := TRVTableItemInfo(rv.GetItem(TableItemNo));
  if table.Rows[0].Count<2 then begin
    Application.MessageBox('Not enough columns!', nil, 0);
    exit;
  end;
  ExtractLinesFromTableColumn(table, 1);
  tb.Max := StreamList.Count-1;
  tb.Visible := True;
  tbChange(tb);
end;

procedure TForm1.tbChange(Sender: TObject);
var Stream: TMemoryStream;
begin
  if (tb.Position<0) or (tb.Position>=StreamList.Count) then
    exit;
  Stream := TMemoryStream(StreamList.Items[tb.Position]);
  Stream.Position := 0;
  rv2.Clear;
  rv2.LoadRVFFromStream(Stream);
  rv2.Format;
end;

end.
