{-------------------------------------------------------------------------------
  Demo: printing documents using TRVReportHelper.
  This demo loads RVF file in rvh.RichView and prints it.
  rvh2 is used to print header: (- page no -).

  Advantages of this method:
  - possibility to make custom complex headers and footers, different for
    different pages;
  - possibility to implement nonstandard layouts, printing several documents
    in different areas on the same page,etc
  Disadvantages of this method:
  - no preview (at least, not with TRVPrintPreview)
  - you need to calculate all margins yourself

 ------------------------------------------------------------------------------

  For example, you can open RVF file created by ActionText (including readme.rvf).
  Do not try to load file from the editor demo (Demos\*\Editors\Editor 1) - it does
  not contain a collection of styles, so it can be opened only by
  applications having that collection of styles.

-------------------------------------------------------------------------------}

unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Printers, Dialogs,
  RVScroll, RichView, PtblRV, RVReport, StdCtrls, RVStyle, CRVPP, RVPP,
  ExtCtrls;

type
  TForm1 = class(TForm)
    Button1: TButton;
    rvh: TRVReportHelper;
    RVStyle1: TRVStyle;
    OpenDialog1: TOpenDialog;
    RVStyle2: TRVStyle;
    RVPrint1: TRVPrint;
    RVPrintPreview1: TRVPrintPreview;
    rvh_fake: TRVReportHelper;
    rvh2: TRVReportHelper;
    procedure FormCreate(Sender: TObject);
    procedure RVPrint1PagePrepaint(Sender: TRVPrint; PageNo: Integer;
      Canvas: TCanvas; Preview: Boolean; PageRect, PrintAreaRect: TRect);
    procedure Button1Click(Sender: TObject);
  private
    { Private declarations }
    FDocWidth: Integer;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

procedure TForm1.FormCreate(Sender: TObject);
begin
  with rvh.RichView do begin
    RVFOptions := [rvfoSavePicturesBody,rvfoSaveControlsBody,rvfoSaveBinary,
      rvfoSaveBack,rvfoLoadBack,rvfoSaveTextStyles,rvfoSaveParaStyles,
      rvfoSaveLayout,rvfoLoadLayout];
    Options := Options + [rvoTagsArePChars];
    RVFParaStylesReadMode := rvf_sInsertMerge;
    RVFTextStylesReadMode := rvf_sInsertMerge;
    Style := RVStyle1;
  end;
  with rvh2.RichView do begin
    Style := RVStyle2;
  end;
  with rvh_fake.RichView do begin
    Style := RVStyle2;
  end;
end;

procedure TForm1.Button1Click(Sender: TObject);
begin
  if OpenDialog1.Execute then begin
    rvh.RichView.Clear;
    rvh.RichView.LoadRTF(OpenDialog1.FileName);

    FDocWidth := -1;
    rvh_fake.RichView.Clear;
    rvh_fake.RichView.AddNL('', 0, 0);
    RVPrint1.AssignSource(rvh_fake.RichView);
    RVPrint1.FormatPages(rvdoALL);
    RVPrintPreview1.UpdateView;
  end;
end;

procedure TForm1.RVPrint1PagePrepaint(Sender: TRVPrint; PageNo: Integer;
  Canvas: TCanvas; Preview: Boolean; PageRect, PrintAreaRect: TRect);
var DocWidth, DocHeight, DocLeft, DocTop, HeaderTop, LineY: Integer;
    pt: TPoint;
begin
  if rvh.RichView.ItemCount=0 then
    exit;

  PageNo := 1;
  DocLeft := Printer.PageWidth div 20; // margins = 5%
  DocWidth := Printer.PageWidth - DocLeft*2;
  HeaderTop := Printer.PageHeight div 20; // margins = 5%
  if DocWidth<>FDocWidth then begin
    rvh.Init(Canvas, DocWidth);
    FDocWidth := DocWidth;
  end;
  while True do begin
    // creating & formatting header
    rvh2.RichView.Clear;
    rvh2.RichView.AddFmt('- %d -', [PageNo], 0, 1);
    rvh2.Init(Canvas, DocWidth);
    rvh2.FormatNextPage(Printer.PageHeight);
    DocTop := HeaderTop+rvh2.EndAt+HeaderTop div 2;
    // formatting next page of document
    DocHeight := Printer.PageHeight-DocTop-HeaderTop;
    if not rvh.FormatNextPage(DocHeight) then
      break;
    // drawing line between header and document
    with Canvas do begin
      Pen.Style := psInsideFrame;
      Pen.Width := 10;
      Pen.Color := clBlack;
      LineY := HeaderTop+rvh2.EndAt+HeaderTop div 4;
      MoveTo(DocLeft*2, LineY);
      LineTo(Printer.PageWidth-DocLeft*2, LineY);
    end;
    // drawing header and document
    SetWindowOrgEx(Canvas.Handle, -DocLeft, -HeaderTop, @pt);
    rvh2.DrawPage(1, Canvas, True, rvh2.EndAt);
    SetWindowOrgEx(Canvas.Handle, -DocLeft, -DocTop, nil);
    rvh.DrawPage(PageNo, Canvas, True, DocHeight);
    SetWindowOrgEx(Canvas.Handle, pt.x, pt.y, nil);
    inc(PageNo);
    break; // only first page
  end;
end;


end.
