unit bugprint;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, RVScroll, RichView, RVEdit, RVStyle,
  PtblRV, RichViewActions, CRVFData, RVTable;

type
  TForm1 = class(TForm)
    RVStyle1: TRVStyle;
    rveMD: TRichViewEdit;
    PrintBtn: TButton;
    Label1: TLabel;
    eRecNumber: TEdit;
    procedure FormActivate(Sender: TObject);
    procedure PrintBtnClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    { Private declarations }
    FRVFStream : TMemoryStream;
    RVPrint : TRVPrint;
    RecNo: integer;
    Finished: boolean;
    procedure LoadTemplateRVF(TemplateName: string);
    function  LoadData(const Code: String): string;
    procedure ReplaceFields(RVData: TCustomRVFormattedData; Frst: integer);
    procedure ReadFile;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

uses dmActions, Printers, MyRVActions;

{$R *.dfm}

procedure TForm1.FormActivate(Sender: TObject);
begin
  LoadTemplateRVF('template.rvf');
  ReadFile;
  rveMD.Format;
  rvActionsResource.rvActionPrintPreview1.Maximized := true;
  rvActionsResource.rvActionPrintPreview1.ActionPrint :=
                                  rvActionsResource.rvActionPrint1;
  rvActionsResource.rvActionPrintPreview1.ExecuteTarget(rveMD);
  PrintBtn.Enabled := true;
end;

procedure TForm1.LoadTemplateRVF(TemplateName: string);
var
  ADevice, ADriver, APort: array[0..79] of Char;
  ADeviceMode: THandle;
  DevMode: PDeviceMode;
  CtrlStream: TMemoryStream;
  MyRVAHdr: TMyHFInfo;
  iPAPERCODE, iPAPERSOURCE, iFIRSTPAGENUMBER, iHDRYMM, iFTRYMM: integer;
begin
  // In the real application a Template is saved in a DB with the rvf and all
  // page setup settings, but here all iXXXX variables, printer orientation,
  // TemplateRVAPRINT, TemplateRVAHEADERINFO, TemplateRVAFOOTERINFO and the
  // Template are being simulated with constants or read from files
  iPAPERCODE:= 1;
  iPAPERSOURCE:= 15;
  iFIRSTPAGENUMBER:= 1;
  iHDRYMM:= 10;
  iFTRYMM:= 10;

  // The stream is necessary to append to rveMD in ReadFile procedure
  FRVFStream.LoadFromFile(TemplateName);
  rveMD.LoadRVFFromStream(FRVFStream);

  CtrlStream:= TMemoryStream.Create;
  try
    CtrlStream.LoadFromFile('TemplateRVAPRINT');
    if CtrlStream.Size > 0 then begin
      CtrlStream.ReadComponent(RVPrint);
    end;
  finally
    CtrlStream.Free;
  end;

  Printer.Orientation := poPortrait;
  Printer.GetPrinter(ADevice,ADriver,APort,ADeviceMode);
  DevMode := PDeviceMode(GlobalLock(ADeviceMode));
  try
    DevMode.dmPaperSize := iPAPERCODE;
    DevMode.dmFields := DevMode.dmFields or DM_PAPERSIZE;
    DevMode.dmDefaultSource := iPAPERSOURCE;
    DevMode.dmFields := DevMode.dmFields or DM_DEFAULTSOURCE;
  finally
    GlobalUnlock(ADeviceMode);
  end;
  Printer.SetPrinter(ADevice,ADriver,APort,ADeviceMode);

  CtrlStream:= TMemoryStream.Create;
  MyRVAHdr := TMyHFInfo.Create(nil);
  try
    CtrlStream.LoadFromFile('TemplateRVAHEADERINFO');
    if CtrlStream.Size > 0 then begin
      CtrlStream.ReadComponent(MyRVAHdr);
      MyRVAHdr.MyAssignTo(RVA_HeaderInfo);
    end;
  finally
    MyRVAHdr.Free;
    CtrlStream.Free;
  end;

  CtrlStream:= TMemoryStream.Create;
  MyRVAHdr := TMyHFInfo.Create(nil);
  try
    CtrlStream.LoadFromFile('TemplateRVAFOOTERINFO');
    if CtrlStream.Size > 0 then begin
      CtrlStream.ReadComponent(MyRVAHdr);
      MyRVAHdr.MyAssignTo(RVA_FooterInfo);
    end;
  finally
    MyRVAHdr.Free;
    CtrlStream.Free;
  end;

  RVA_FirstPageNumber:= iFIRSTPAGENUMBER;
  RVPrint.HeaderYMM := iHDRYMM;
  RVPrint.HeaderYMM := iFTRYMM;
end;

function TForm1.LoadData(const Code: String): string;
begin
  result := '';
  if not Finished then  begin
    case Code[1] of
      'R': // fields
        case Code[2] of
          '1': result := 'field1 of record #' + IntToStr(RecNo);
          '2': result := 'field2 of record #' + IntToStr(RecNo);
          '3': result := 'field3 of record #' + IntToStr(RecNo);
          else
            result := Code;
        end;
      'N':  // Next record
        begin
          Inc(RecNo);
          if (RecNo > StrToIntDef(eRecNumber.Text,10)) then
            Finished := true;
        end;
      else
        result := Code;
    end;
  end;
end;

procedure TForm1.ReplaceFields(RVData: TCustomRVFormattedData; Frst: integer);
var
    table: TRVTableItemInfo;
    i, r,c: Integer;
    code: string;
begin
  for i := Frst to RVData.ItemCount-1 do begin
    case RVData.GetItemStyle(i) of
      rvsTable:
        begin
          // recursive call for table cells
          table := TRVTableItemInfo(RVData.GetItem(i));
          for r := 0 to table.Rows.Count-1 do
            for c := 0 to table.Rows[r].Count-1 do
              if table.Cells[r,c]<>nil then
                ReplaceFields(table.Cells[r,c],0);
        end;
      else begin
          code := Pchar(RVData.GetItemTag(i));
          if code <> '' then begin
            RVData.SetItemText(i,LoadData(code));
          end;
      end;
    end; // case
  end; // for
  RVData.Normalize;
end;

procedure TForm1.ReadFile;
var
  rveItemCount: integer;
begin
  RecNo := 1;
  Finished := false;
  rveItemCount := 0;
  repeat
    ReplaceFields(rveMD.RVData, rveItemCount);
    if not Finished then begin
      rveItemCount := rveMD.ItemCount;
      FRVFStream.Position := 0;
      rveMD.InsertRVFFromStream(FRVFStream,rveItemCount);
    end;
  until Finished;
end;

procedure TForm1.PrintBtnClick(Sender: TObject);
begin
  rvActionsResource.rvActionPrint1.ExecuteTarget(rveMD);
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  FRVFStream := TMemoryStream.Create;
  RVPrint := TRVPrint.Create(self);
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  FRVFStream.free;
  RVPrint.free;
end;

end.
